<?php

namespace Drupal\xray_audit\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Url;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Displays a given  entity using a given display mode.
 *
 * Used for example purposes, to display an entity on different incarnations.
 *
 * @package Drupal\xray_audit\Controller
 */
class XrayAuditDisplayModeExampleController extends ControllerBase {

  /**
   * Display a entity in a given view mode.
   *
   * @param string $entity_type
   *   The view mode.
   * @param string $entity_id
   *   The entity type.
   * @param string $view_mode
   *   The entity id.
   */
  public function displayEntity(string $entity_type, string $entity_id, string $view_mode) {
    $entity = $this->entityTypeManager()->getStorage($entity_type)->load($entity_id);
    $response = $this->entityTypeManager()->getViewBuilder($entity_type)->view($entity, $view_mode);

    return $response;
  }

  /**
   * Display an entity in a given view mode.
   *
   * @param string $entity_type
   *   The entity type.
   * @param string $entity_bundle
   *   The entity bundle type.
   * @param string $view_mode
   *   The view mode.
   */
  public function displayEntityPopupContent(string $entity_type, string $entity_bundle, string $view_mode) {
    $id = $this->getExampleId($entity_type, $entity_bundle);

    $build = [
      '#theme' => 'xray_audit_popup',
    ];

    if (!$id) {
      $build['#content'] = [
        '#type' => 'markup',
        '#markup' => $this->t('No examples available.'),
      ];
      $build['#attributes']['class'] = ['xray-audit--message'];
    }
    elseif ($entity_type === 'node' && ($view_mode === 'default' || $view_mode === 'full')) {
      $url = Url::fromRoute('entity.node.canonical', ['node' => $id]);

      return new RedirectResponse($url->toString());
    }
    else {
      $build['#attributes']['class'] = ['xray-audit--iframe'];
      $build['#iframe_url'] = Url::fromRoute('xray_audit.display_mode_example', [
        'entity_type' => $entity_type,
        'entity_id' => $id,
        'view_mode' => $view_mode,
      ]);
    }
    return $build;
  }

  /**
   * Provide random entity id.
   *
   * @param string $entity_type
   *   Entity name.
   * @param string $entity_bundle
   *   Entity type name.
   */
  protected function getExampleId(string $entity_type, string $entity_bundle) {
    $query = $this->entityTypeManager()->getStorage($entity_type)->getQuery();
    $bundle_column = 'type';
    $sort_column = 'changed';

    if ($entity_type === 'media') {
      $bundle_column = 'bundle';
    }
    elseif ($entity_type === 'taxonomy_term') {
      $bundle_column = 'vid';
    }

    if ($entity_type === 'paragraph') {
      $sort_column = 'created';
    }

    $ids = $query->accessCheck(TRUE)
      ->condition($bundle_column, $entity_bundle)
      ->range(0, 100)
      ->sort($sort_column, 'DESC')
      ->execute();

    if (count($ids) > 0) {
      return $ids[array_rand($ids)];
    }
    return FALSE;
  }

}
